# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.internal_server_error import InternalServerError
from ...errors.payment_required_error import PaymentRequiredError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.error import Error
from ...types.mms_content import MmsContent
from ...types.rcs_validate_content import RcsValidateContent
from ...types.sms_content import SmsContent
from .types.blast_mms_options import BlastMmsOptions
from .types.blast_mms_response import BlastMmsResponse
from .types.blast_rcs_options import BlastRcsOptions
from .types.blast_rcs_response import BlastRcsResponse
from .types.blast_sms_options import BlastSmsOptions
from .types.blast_sms_response import BlastSmsResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBlastClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def sms(
        self,
        *,
        audience_id: str,
        senders: typing.Sequence[str],
        message: SmsContent,
        options: typing.Optional[BlastSmsOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BlastSmsResponse]:
        """
        Send an SMS message to all contacts in an audience. <br>

        Messages are distributed evenly across the provided sender phone numbers. <br>

        Use the optional `schedule` parameter in `options` to schedule the blast for future delivery. When scheduled, the response will contain a `scheduleId` instead of blast details.

        Parameters
        ----------
        audience_id : str
            The audience ID to send the blast to. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`. <br>

            You can create an audience via [the dashboard](https://app.pinnacle.sh/dashboard/audiences) or [API](/api-reference/audiences/create).

        senders : typing.Sequence[str]
            Array of phone numbers to send from in E.164 format. <br>

            Messages will be distributed evenly across these senders.

            > **Note:** Sandbox numbers cannot be used for blasts.

            **Limit:** 1 min

        message : SmsContent

        options : typing.Optional[BlastSmsOptions]
            Additional settings to customize SMS blast delivery.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BlastSmsResponse]
            Successfully queued or scheduled the SMS blast. <br>

            - **Immediate blast**: Returns blast details including total recipients, messages, segments, and cost.
            - **Scheduled blast**: Returns a `scheduleId` and the schedule configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/blast/sms",
            method="POST",
            json={
                "audienceId": audience_id,
                "senders": senders,
                "message": convert_and_respect_annotation_metadata(
                    object_=message, annotation=SmsContent, direction="write"
                ),
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=BlastSmsOptions, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BlastSmsResponse,
                    parse_obj_as(
                        type_=BlastSmsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def mms(
        self,
        *,
        audience_id: str,
        senders: typing.Sequence[str],
        message: MmsContent,
        options: typing.Optional[BlastMmsOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BlastMmsResponse]:
        """
        Send an MMS message to all contacts in an audience. <br>

        Messages are distributed evenly across the provided sender phone numbers. <br>

        Use the optional `schedule` parameter in `options` to schedule the blast for future delivery. When scheduled, the response will contain a `scheduleId` instead of blast details.

        Parameters
        ----------
        audience_id : str
            The audience ID to send the blast to. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`. <br>

            You can create an audience via [the dashboard](https://app.pinnacle.sh/dashboard/audiences) or [API](/api-reference/audiences/create).

        senders : typing.Sequence[str]
            Array of phone numbers to send from in E.164 format. <br>

            Messages will be distributed evenly across these senders.

            > **Note:** Sandbox numbers cannot be used for blasts.

            **Limit:** 1 min

        message : MmsContent

        options : typing.Optional[BlastMmsOptions]
            Additional settings to customize MMS blast delivery.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BlastMmsResponse]
            Successfully queued or scheduled the MMS blast. <br>

            - **Immediate blast**: Returns blast details including total recipients, messages, segments, and cost.
            - **Scheduled blast**: Returns a `scheduleId` and the schedule configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/blast/mms",
            method="POST",
            json={
                "audienceId": audience_id,
                "senders": senders,
                "message": convert_and_respect_annotation_metadata(
                    object_=message, annotation=MmsContent, direction="write"
                ),
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=BlastMmsOptions, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BlastMmsResponse,
                    parse_obj_as(
                        type_=BlastMmsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def rcs(
        self,
        *,
        audience_id: str,
        senders: typing.Sequence[str],
        message: RcsValidateContent,
        options: typing.Optional[BlastRcsOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BlastRcsResponse]:
        """
        Send an RCS message to all contacts in an audience. <br>

        Messages are distributed evenly across the provided RCS agents for load balancing. Requires active RCS agents and recipient devices that support RCS Business Messaging. <br>

        Use the optional `schedule` parameter in `options` to schedule the blast for future delivery. When scheduled, the response will contain a `scheduleId` instead of blast details.

        Parameters
        ----------
        audience_id : str
            The audience ID to send the blast to. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`. <br>

            You can create an audience via [the dashboard](https://app.pinnacle.sh/dashboard/audiences) or [API](/api-reference/audiences/create).

        senders : typing.Sequence[str]
            Array of RCS agent IDs to send from. Each must be prefixed with `agent_`. <br>

            Messages will be evenly distributed across these agents.

            > **Note:** Test agents cannot be used for blasts.

            **Limit:** 1 min

        message : RcsValidateContent

        options : typing.Optional[BlastRcsOptions]
            Configure how your RCS blast is sent and tracked.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BlastRcsResponse]
            Successfully queued or scheduled the RCS blast. <br>

            - **Immediate blast**: Returns blast details including total recipients, messages, segments, and cost.
            - **Scheduled blast**: Returns a `scheduleId` and the schedule configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/blast/rcs",
            method="POST",
            json={
                "audienceId": audience_id,
                "senders": senders,
                "message": convert_and_respect_annotation_metadata(
                    object_=message, annotation=RcsValidateContent, direction="write"
                ),
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=BlastRcsOptions, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BlastRcsResponse,
                    parse_obj_as(
                        type_=BlastRcsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBlastClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def sms(
        self,
        *,
        audience_id: str,
        senders: typing.Sequence[str],
        message: SmsContent,
        options: typing.Optional[BlastSmsOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BlastSmsResponse]:
        """
        Send an SMS message to all contacts in an audience. <br>

        Messages are distributed evenly across the provided sender phone numbers. <br>

        Use the optional `schedule` parameter in `options` to schedule the blast for future delivery. When scheduled, the response will contain a `scheduleId` instead of blast details.

        Parameters
        ----------
        audience_id : str
            The audience ID to send the blast to. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`. <br>

            You can create an audience via [the dashboard](https://app.pinnacle.sh/dashboard/audiences) or [API](/api-reference/audiences/create).

        senders : typing.Sequence[str]
            Array of phone numbers to send from in E.164 format. <br>

            Messages will be distributed evenly across these senders.

            > **Note:** Sandbox numbers cannot be used for blasts.

            **Limit:** 1 min

        message : SmsContent

        options : typing.Optional[BlastSmsOptions]
            Additional settings to customize SMS blast delivery.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BlastSmsResponse]
            Successfully queued or scheduled the SMS blast. <br>

            - **Immediate blast**: Returns blast details including total recipients, messages, segments, and cost.
            - **Scheduled blast**: Returns a `scheduleId` and the schedule configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/blast/sms",
            method="POST",
            json={
                "audienceId": audience_id,
                "senders": senders,
                "message": convert_and_respect_annotation_metadata(
                    object_=message, annotation=SmsContent, direction="write"
                ),
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=BlastSmsOptions, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BlastSmsResponse,
                    parse_obj_as(
                        type_=BlastSmsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def mms(
        self,
        *,
        audience_id: str,
        senders: typing.Sequence[str],
        message: MmsContent,
        options: typing.Optional[BlastMmsOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BlastMmsResponse]:
        """
        Send an MMS message to all contacts in an audience. <br>

        Messages are distributed evenly across the provided sender phone numbers. <br>

        Use the optional `schedule` parameter in `options` to schedule the blast for future delivery. When scheduled, the response will contain a `scheduleId` instead of blast details.

        Parameters
        ----------
        audience_id : str
            The audience ID to send the blast to. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`. <br>

            You can create an audience via [the dashboard](https://app.pinnacle.sh/dashboard/audiences) or [API](/api-reference/audiences/create).

        senders : typing.Sequence[str]
            Array of phone numbers to send from in E.164 format. <br>

            Messages will be distributed evenly across these senders.

            > **Note:** Sandbox numbers cannot be used for blasts.

            **Limit:** 1 min

        message : MmsContent

        options : typing.Optional[BlastMmsOptions]
            Additional settings to customize MMS blast delivery.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BlastMmsResponse]
            Successfully queued or scheduled the MMS blast. <br>

            - **Immediate blast**: Returns blast details including total recipients, messages, segments, and cost.
            - **Scheduled blast**: Returns a `scheduleId` and the schedule configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/blast/mms",
            method="POST",
            json={
                "audienceId": audience_id,
                "senders": senders,
                "message": convert_and_respect_annotation_metadata(
                    object_=message, annotation=MmsContent, direction="write"
                ),
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=BlastMmsOptions, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BlastMmsResponse,
                    parse_obj_as(
                        type_=BlastMmsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def rcs(
        self,
        *,
        audience_id: str,
        senders: typing.Sequence[str],
        message: RcsValidateContent,
        options: typing.Optional[BlastRcsOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BlastRcsResponse]:
        """
        Send an RCS message to all contacts in an audience. <br>

        Messages are distributed evenly across the provided RCS agents for load balancing. Requires active RCS agents and recipient devices that support RCS Business Messaging. <br>

        Use the optional `schedule` parameter in `options` to schedule the blast for future delivery. When scheduled, the response will contain a `scheduleId` instead of blast details.

        Parameters
        ----------
        audience_id : str
            The audience ID to send the blast to. This identifier is a string that always begins with the prefix `aud_`, for example: `aud_abc123`. <br>

            You can create an audience via [the dashboard](https://app.pinnacle.sh/dashboard/audiences) or [API](/api-reference/audiences/create).

        senders : typing.Sequence[str]
            Array of RCS agent IDs to send from. Each must be prefixed with `agent_`. <br>

            Messages will be evenly distributed across these agents.

            > **Note:** Test agents cannot be used for blasts.

            **Limit:** 1 min

        message : RcsValidateContent

        options : typing.Optional[BlastRcsOptions]
            Configure how your RCS blast is sent and tracked.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BlastRcsResponse]
            Successfully queued or scheduled the RCS blast. <br>

            - **Immediate blast**: Returns blast details including total recipients, messages, segments, and cost.
            - **Scheduled blast**: Returns a `scheduleId` and the schedule configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/blast/rcs",
            method="POST",
            json={
                "audienceId": audience_id,
                "senders": senders,
                "message": convert_and_respect_annotation_metadata(
                    object_=message, annotation=RcsValidateContent, direction="write"
                ),
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=BlastRcsOptions, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BlastRcsResponse,
                    parse_obj_as(
                        type_=BlastRcsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.internal_server_error import InternalServerError
from ...errors.not_found_error import NotFoundError
from ...errors.not_implemented_error import NotImplementedError
from ...errors.payment_required_error import PaymentRequiredError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.error import Error
from ...types.rcs_validate_content import RcsValidateContent
from ...types.rcs_validation_result import RcsValidationResult
from ...types.rich_message import RichMessage
from ...types.send_typing_indicator_response import SendTypingIndicatorResponse
from .types.send_rich_message_response import SendRichMessageResponse
from .types.send_typing_indicator_schema_options import SendTypingIndicatorSchemaOptions

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRcsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def send(
        self, *, request: RichMessage, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SendRichMessageResponse]:
        """
        Send a RCS message immediately or schedule it for future delivery. <br>

        Requires an active RCS agent and recipient devices that support RCS Business Messaging.

        Parameters
        ----------
        request : RichMessage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SendRichMessageResponse]
            Successfully sent or scheduled the message.

            Use our [/messages/:id](./get) endpoint to track your message.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/send/rcs",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=RichMessage, direction="write"),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SendRichMessageResponse,
                    parse_obj_as(
                        type_=SendRichMessageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def send_typing(
        self,
        *,
        agent_id: str,
        to: str,
        options: typing.Optional[SendTypingIndicatorSchemaOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SendTypingIndicatorResponse]:
        """
        Send a typing indicator from an RCS agent to a recipient.

        This endpoint allows RCS agents to display a typing indicator to recipients. The indicator is a message bubble with animated typing dots like this: <img src="https://server.trypinnacle.app/storage/v1/object/public/pinnacle-public-assets/ios-typing-indicator.png" alt="Typing Indicator" style="display: inline; height: 1.5em; vertical-align: middle; margin: 0 4px;" />

        **Use Case:** Typing indicators are especially useful for providing feedback to users while the agent is thinking or generating a response that may take some time, creating a more engaging conversational experience.

        **Expiration:** Typing indicators automatically expire after around 20 seconds or when the agent sends a message, whichever comes first.

        **Frequency:** You can send typing indicators as many times as needed, though only one will be displayed at a time. Sending multiple typing indicators will extend the duration of the current indicator.

        > **Note:** Typing indicators are best-effort hints, not delivery-guaranteed state. The platform is allowed to coalesce or drop them, and the client UI decides when to show/hide.

        Parameters
        ----------
        agent_id : str
            The unique identifier of the RCS agent sending the typing indicator. <br>

            Format: `agent_` followed by alphanumeric characters (e.g., `agent_pinnacle`).

        to : str
            The recipient's phone number in E.164 format. <br>

            Must include country code with a leading plus sign (e.g., `+14155551234`).

        options : typing.Optional[SendTypingIndicatorSchemaOptions]
            Configure how your typing indicator is sent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SendTypingIndicatorResponse]
            Successfully sent the typing indicator.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/send/typing",
            method="POST",
            json={
                "agentId": agent_id,
                "to": to,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=SendTypingIndicatorSchemaOptions, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SendTypingIndicatorResponse,
                    parse_obj_as(
                        type_=SendTypingIndicatorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def validate(
        self, *, request: RcsValidateContent, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RcsValidationResult]:
        """
        Validate RCS message content without sending it.

        Parameters
        ----------
        request : RcsValidateContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RcsValidationResult]
            Returns validation results.
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/validate/rcs",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=RcsValidateContent, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RcsValidationResult,
                    parse_obj_as(
                        type_=RcsValidationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRcsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def send(
        self, *, request: RichMessage, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SendRichMessageResponse]:
        """
        Send a RCS message immediately or schedule it for future delivery. <br>

        Requires an active RCS agent and recipient devices that support RCS Business Messaging.

        Parameters
        ----------
        request : RichMessage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SendRichMessageResponse]
            Successfully sent or scheduled the message.

            Use our [/messages/:id](./get) endpoint to track your message.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/send/rcs",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=RichMessage, direction="write"),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SendRichMessageResponse,
                    parse_obj_as(
                        type_=SendRichMessageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def send_typing(
        self,
        *,
        agent_id: str,
        to: str,
        options: typing.Optional[SendTypingIndicatorSchemaOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SendTypingIndicatorResponse]:
        """
        Send a typing indicator from an RCS agent to a recipient.

        This endpoint allows RCS agents to display a typing indicator to recipients. The indicator is a message bubble with animated typing dots like this: <img src="https://server.trypinnacle.app/storage/v1/object/public/pinnacle-public-assets/ios-typing-indicator.png" alt="Typing Indicator" style="display: inline; height: 1.5em; vertical-align: middle; margin: 0 4px;" />

        **Use Case:** Typing indicators are especially useful for providing feedback to users while the agent is thinking or generating a response that may take some time, creating a more engaging conversational experience.

        **Expiration:** Typing indicators automatically expire after around 20 seconds or when the agent sends a message, whichever comes first.

        **Frequency:** You can send typing indicators as many times as needed, though only one will be displayed at a time. Sending multiple typing indicators will extend the duration of the current indicator.

        > **Note:** Typing indicators are best-effort hints, not delivery-guaranteed state. The platform is allowed to coalesce or drop them, and the client UI decides when to show/hide.

        Parameters
        ----------
        agent_id : str
            The unique identifier of the RCS agent sending the typing indicator. <br>

            Format: `agent_` followed by alphanumeric characters (e.g., `agent_pinnacle`).

        to : str
            The recipient's phone number in E.164 format. <br>

            Must include country code with a leading plus sign (e.g., `+14155551234`).

        options : typing.Optional[SendTypingIndicatorSchemaOptions]
            Configure how your typing indicator is sent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SendTypingIndicatorResponse]
            Successfully sent the typing indicator.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/send/typing",
            method="POST",
            json={
                "agentId": agent_id,
                "to": to,
                "options": convert_and_respect_annotation_metadata(
                    object_=options, annotation=SendTypingIndicatorSchemaOptions, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SendTypingIndicatorResponse,
                    parse_obj_as(
                        type_=SendTypingIndicatorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def validate(
        self, *, request: RcsValidateContent, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RcsValidationResult]:
        """
        Validate RCS message content without sending it.

        Parameters
        ----------
        request : RcsValidateContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RcsValidationResult]
            Returns validation results.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/validate/rcs",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=RcsValidateContent, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RcsValidationResult,
                    parse_obj_as(
                        type_=RcsValidationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .advanced_phone_information_carrier import AdvancedPhoneInformationCarrier
from .advanced_phone_information_contact import AdvancedPhoneInformationContact
from .advanced_phone_information_location import AdvancedPhoneInformationLocation
from .advanced_phone_information_type import AdvancedPhoneInformationType
from .number_format import NumberFormat


class AdvancedPhoneInformation(UniversalBaseModel):
    """
    Detailed phone number analysis including validation status, classification with fraud risk,
    precise geographic data, carrier intelligence, and enhanced contact information.
    Provides comprehensive insights for risk assessment, compliance, and advanced usage scenarios.
    """

    is_valid: typing_extensions.Annotated[bool, FieldMetadata(alias="isValid")] = pydantic.Field()
    """
    Indicates whether the phone number is valid and capable of receiving communications.
    """

    type: AdvancedPhoneInformationType = pydantic.Field()
    """
    Detailed classification including fraud risk and security recommendations.
    """

    formats: NumberFormat = pydantic.Field()
    """
    Different standardized ways the phone number can be formatted for display or dialing.
    """

    location: AdvancedPhoneInformationLocation = pydantic.Field()
    """
    Comprehensive geographic and administrative location data with precise coordinates
    and timezone information for accurate localization.
    """

    carrier: AdvancedPhoneInformationCarrier = pydantic.Field()
    """
    Detailed carrier information.
    """

    contact: typing.Optional[AdvancedPhoneInformationContact] = pydantic.Field(default=None)
    """
    Enhanced contact information associated with the phone number.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from .extended_brand import ExtendedBrand
from .vetting_feedback import VettingFeedback
from .vetting_history import VettingHistory


class ExtendedBrandWithVetting(ExtendedBrand):
    last_tcr_vetting_date: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="lastTcrVettingDate")
    ] = pydantic.Field(default=None)
    """
    ISO 8601 formatted timestamp of the last TCR vetting.
    """

    tcr_feedback: typing_extensions.Annotated[
        typing.Optional[typing.List[VettingFeedback]], FieldMetadata(alias="tcrFeedback")
    ] = pydantic.Field(default=None)
    """
    Feedback from The Campaign Registry if TCR vetting was performed.
    """

    vetting_history: typing_extensions.Annotated[typing.List[VettingHistory], FieldMetadata(alias="vettingHistory")] = (
        pydantic.Field()
    )
    """
    History of all external vetting attempts for this brand.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....types.scheduled_message import ScheduledMessage
from ....types.sent_mms_details import SentMmsDetails

MmsSendResponse = typing.Union[SentMmsDetails, ScheduledMessage]



# This file was auto-generated by Fern from our API Definition.

import typing

PhoneNumberEnum = typing.Union[
    typing.Literal["LANDLINE", "MOBILE", "SATELLITE", "PREMIUM", "PAGING", "SPECIAL", "TOLL_FREE", "UNKNOWN"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_card_geo import VCardGeo
from .v_card_name import VCardName
from .v_card_organization import VCardOrganization
from .vcard_address import VcardAddress
from .vcard_email import VcardEmail
from .vcard_phone import VcardPhone


class Vcard(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the contact. This identifier is a string that always begins with the prefix `cc_`, for example: `cc_1234567890`.
    """

    formatted_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="formattedName")] = (
        pydantic.Field(default=None)
    )
    """
    Full display name for the vCard.
    """

    name: typing.Optional[VCardName] = pydantic.Field(default=None)
    """
    Structured name components.
    """

    nickname: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Nicknames or aliases.
    """

    birthday: typing.Optional[str] = pydantic.Field(default=None)
    """
    Birthday in ISO 8601 date format (YYYY-MM-DD).
    """

    addresses: typing.Optional[typing.List[VcardAddress]] = pydantic.Field(default=None)
    """
    Physical addresses.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Website URL.
    """

    phones: typing.Optional[typing.List[VcardPhone]] = pydantic.Field(default=None)
    """
    Phone numbers.
    """

    emails: typing.Optional[typing.List[VcardEmail]] = pydantic.Field(default=None)
    """
    Email addresses.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    Timezone (e.g., "America/New_York").
    """

    geo: typing.Optional[VCardGeo] = pydantic.Field(default=None)
    """
    Geographic coordinates.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Job title or position.
    """

    role: typing.Optional[str] = pydantic.Field(default=None)
    """
    Role or function within the organization.
    """

    organization: typing.Optional[VCardOrganization] = pydantic.Field(default=None)
    """
    Organization or company information.
    """

    categories: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Categories or tags for organizing contacts.
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    Additional notes or comments.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class AdvancedPhoneInformationCarrier(UniversalBaseModel):
    """
    Detailed carrier information.
    """

    name: str = pydantic.Field()
    """
    Carrier or service provider name as registered with telecom authorities.
    """

    normalized_carrier: typing_extensions.Annotated[str, FieldMetadata(alias="normalizedCarrier")] = pydantic.Field()
    """
    Standardized carrier name used across data sources.
    """

    mcc: str = pydantic.Field()
    """
    Mobile Country Code - 3-digit identifier assigned by ITU-T for the country.
    Used in GSM, UMTS, and LTE networks for international roaming and identification.
    """

    mnc: str = pydantic.Field()
    """
    Mobile Network Code - 2 or 3-digit identifier for the specific carrier within the country.
    Combined with MCC provides unique global identification of the mobile network.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .mms_validation_response_segments import MmsValidationResponseSegments


class MmsValidationResult(UniversalBaseModel):
    segments: MmsValidationResponseSegments = pydantic.Field()
    """
    Details about how the MMS content would be segmented for delivery.
    """

    total: float = pydantic.Field()
    """
    Total price of this message. Calculated by multiplying the number of segments by the unit cost.
    """

    unit: float = pydantic.Field()
    """
    Cost of each segment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CompanyTypeEnum = typing.Union[
    typing.Literal["GOVERNMENT", "NON_PROFIT", "PRIVATE_PROFIT", "PUBLIC_PROFIT", "SOLE_PROPRIETOR"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .link_click_event import LinkClickEvent
from .shortened_url import ShortenedUrl


class ShortenedUrlWithClickData(ShortenedUrl):
    clicks: typing.List[LinkClickEvent] = pydantic.Field()
    """
    Array of click analytics data for this URL. <br>
    
    Click data will be empty if no one has visited your link yet.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata
from .toll_free_campaign_help_keywords import TollFreeCampaignHelpKeywords
from .toll_free_campaign_opt_in_keywords import TollFreeCampaignOptInKeywords


class TollFreeCampaignKeywords(UniversalBaseModel):
    """
    Keyword response configuration.
    """

    help: typing_extensions.Annotated[typing.Optional[TollFreeCampaignHelpKeywords], FieldMetadata(alias="HELP")] = (
        pydantic.Field(default=None)
    )
    """
    Help keyword settings.
    """

    opt_in: typing_extensions.Annotated[
        typing.Optional[TollFreeCampaignOptInKeywords], FieldMetadata(alias="OPT_IN")
    ] = pydantic.Field(default=None)
    """
    Opt-in keyword settings.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.webhook_result import WebhookResult
from .raw_client import AsyncRawWebhooksClient, RawWebhooksClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WebhooksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWebhooksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWebhooksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWebhooksClient
        """
        return self._raw_client

    def get(
        self, *, identifiers: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> WebhookResult:
        """
        Retrieve all webhook that are set up to receive events for specific URLs or phone numbers.

        Parameters
        ----------
        identifiers : typing.Sequence[str]
            List of URLs or phone numbers in E.164 format that the webhook is attached to. <br><br>
            **Limit:** 1 min

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookResult
            Returns all webhooks matching the search criteria. <br>

            Results are sorted by ID and duplicates are removed. This identifier is a string that always begins with the prefix `wh_`, for example: `wh_1234567890`.

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.webhooks.get(
            identifiers=[
                "https://www.pinnacle.sh/payment",
                "+14155678901",
                "https://www.pinnacle.sh/sms-callback",
                "+14153456659",
            ],
        )
        """
        _response = self._raw_client.get(identifiers=identifiers, request_options=request_options)
        return _response.data


class AsyncWebhooksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWebhooksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWebhooksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWebhooksClient
        """
        return self._raw_client

    async def get(
        self, *, identifiers: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> WebhookResult:
        """
        Retrieve all webhook that are set up to receive events for specific URLs or phone numbers.

        Parameters
        ----------
        identifiers : typing.Sequence[str]
            List of URLs or phone numbers in E.164 format that the webhook is attached to. <br><br>
            **Limit:** 1 min

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookResult
            Returns all webhooks matching the search criteria. <br>

            Results are sorted by ID and duplicates are removed. This identifier is a string that always begins with the prefix `wh_`, for example: `wh_1234567890`.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.webhooks.get(
                identifiers=[
                    "https://www.pinnacle.sh/payment",
                    "+14155678901",
                    "https://www.pinnacle.sh/sms-callback",
                    "+14153456659",
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(identifiers=identifiers, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.v_card_data import VCardData
from ...types.v_card_geo import VCardGeo
from ...types.v_card_name import VCardName
from ...types.v_card_organization import VCardOrganization
from ...types.vcard_address import VcardAddress
from ...types.vcard_email import VcardEmail
from ...types.vcard_phone import VcardPhone
from ...types.vcard_resource import VcardResource
from .raw_client import AsyncRawContactCardClient, RawContactCardClient
from .types.get_v_card_schema_options import GetVCardSchemaOptions

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ContactCardClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawContactCardClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawContactCardClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawContactCardClient
        """
        return self._raw_client

    def get(
        self,
        *,
        id: str,
        options: typing.Optional[GetVCardSchemaOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VCardData:
        """
        Retrieve contact information as a vCard and get a presigned URL to download the file. Contact cards can be sent [via MMS](/api-reference/messages/send-mms) as a media file.

        Parameters
        ----------
        id : str
            The unique identifier of the contact. This identifier is a string that always begins with the prefix `cc_`, for example: `cc_1234567890`.

        options : typing.Optional[GetVCardSchemaOptions]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VCardData
            Returns complete contact information.

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.tools.contact_card.get(
            id="cc_1234567890",
        )
        """
        _response = self._raw_client.get(id=id, options=options, request_options=request_options)
        return _response.data

    def upsert(
        self,
        *,
        photo: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        formatted_name: typing.Optional[str] = OMIT,
        name: typing.Optional[VCardName] = OMIT,
        nickname: typing.Optional[typing.Sequence[str]] = OMIT,
        birthday: typing.Optional[str] = OMIT,
        addresses: typing.Optional[typing.Sequence[VcardAddress]] = OMIT,
        url: typing.Optional[str] = OMIT,
        phones: typing.Optional[typing.Sequence[VcardPhone]] = OMIT,
        emails: typing.Optional[typing.Sequence[VcardEmail]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        geo: typing.Optional[VCardGeo] = OMIT,
        title: typing.Optional[str] = OMIT,
        role: typing.Optional[str] = OMIT,
        organization: typing.Optional[VCardOrganization] = OMIT,
        categories: typing.Optional[typing.Sequence[str]] = OMIT,
        note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VcardResource:
        """
        Create a new contact card or updates an existing one with full vCard data. Contact cards can be sent [via MMS](/api-reference/messages/send-mms) as a media file.

        Parameters
        ----------
        photo : typing.Optional[str]
            Contact's photo

        id : typing.Optional[str]
            The unique identifier of the contact. This identifier is a string that always begins with the prefix `cc_`, for example: `cc_1234567890`.

        formatted_name : typing.Optional[str]
            Full display name for the vCard.

        name : typing.Optional[VCardName]
            Structured name components.

        nickname : typing.Optional[typing.Sequence[str]]
            Nicknames or aliases.

        birthday : typing.Optional[str]
            Birthday in ISO 8601 date format (YYYY-MM-DD).

        addresses : typing.Optional[typing.Sequence[VcardAddress]]
            Physical addresses.

        url : typing.Optional[str]
            Website URL.

        phones : typing.Optional[typing.Sequence[VcardPhone]]
            Phone numbers.

        emails : typing.Optional[typing.Sequence[VcardEmail]]
            Email addresses.

        timezone : typing.Optional[str]
            Timezone (e.g., "America/New_York").

        geo : typing.Optional[VCardGeo]
            Geographic coordinates.

        title : typing.Optional[str]
            Job title or position.

        role : typing.Optional[str]
            Role or function within the organization.

        organization : typing.Optional[VCardOrganization]
            Organization or company information.

        categories : typing.Optional[typing.Sequence[str]]
            Categories or tags for organizing contacts.

        note : typing.Optional[str]
            Additional notes or comments.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VcardResource
            Successfully created/updated contact card.

        Examples
        --------
        from rcs import (
            Pinnacle,
            VcardAddress,
            VcardEmail,
            VCardGeo,
            VCardName,
            VCardOrganization,
            VcardPhone,
        )

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.tools.contact_card.upsert(
            id="cc_1234567890",
            formatted_name="Jane Smith",
            name=VCardName(
                family_name="Smith",
                given_name="Jane",
                additional_names=["A."],
                honorific_prefixes=["Dr."],
                honorific_suffixes=["PhD"],
            ),
            nickname=["Janie"],
            birthday="1990-02-15",
            addresses=[
                VcardAddress(
                    country_name="USA",
                    extended_address="Apt. 4B",
                    locality="Anytown",
                    postal_code="90210",
                    post_office_box="PO Box 123",
                    region="CA",
                    street_address="123 Main St",
                    type=["HOME", "PREF"],
                )
            ],
            url="https://app.pinnacle.sh",
            phones=[
                VcardPhone(
                    type=["CELL"],
                    value="+15551234567",
                )
            ],
            emails=[
                VcardEmail(
                    type=["INTERNET"],
                    value="jane.smith@example.com",
                )
            ],
            timezone="America/Los_Angeles",
            geo=VCardGeo(
                latitude=34.0522,
                longitude=-118.2437,
            ),
            title="Engineer",
            role="Developer",
            organization=VCardOrganization(
                name="Acme Co",
                units=["Engineering", "R&D"],
            ),
            categories=["Friend", "Colleague"],
            note="Test contact entry",
            photo="https://fastly.picsum.photos/id/853/200/200.jpg?hmac=f4LF-tVBBnJb9PQAVEO8GCTGWgLUnxQLw44rUofE6mQ",
        )
        """
        _response = self._raw_client.upsert(
            photo=photo,
            id=id,
            formatted_name=formatted_name,
            name=name,
            nickname=nickname,
            birthday=birthday,
            addresses=addresses,
            url=url,
            phones=phones,
            emails=emails,
            timezone=timezone,
            geo=geo,
            title=title,
            role=role,
            organization=organization,
            categories=categories,
            note=note,
            request_options=request_options,
        )
        return _response.data


class AsyncContactCardClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawContactCardClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawContactCardClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawContactCardClient
        """
        return self._raw_client

    async def get(
        self,
        *,
        id: str,
        options: typing.Optional[GetVCardSchemaOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VCardData:
        """
        Retrieve contact information as a vCard and get a presigned URL to download the file. Contact cards can be sent [via MMS](/api-reference/messages/send-mms) as a media file.

        Parameters
        ----------
        id : str
            The unique identifier of the contact. This identifier is a string that always begins with the prefix `cc_`, for example: `cc_1234567890`.

        options : typing.Optional[GetVCardSchemaOptions]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VCardData
            Returns complete contact information.

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.contact_card.get(
                id="cc_1234567890",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id=id, options=options, request_options=request_options)
        return _response.data

    async def upsert(
        self,
        *,
        photo: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        formatted_name: typing.Optional[str] = OMIT,
        name: typing.Optional[VCardName] = OMIT,
        nickname: typing.Optional[typing.Sequence[str]] = OMIT,
        birthday: typing.Optional[str] = OMIT,
        addresses: typing.Optional[typing.Sequence[VcardAddress]] = OMIT,
        url: typing.Optional[str] = OMIT,
        phones: typing.Optional[typing.Sequence[VcardPhone]] = OMIT,
        emails: typing.Optional[typing.Sequence[VcardEmail]] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        geo: typing.Optional[VCardGeo] = OMIT,
        title: typing.Optional[str] = OMIT,
        role: typing.Optional[str] = OMIT,
        organization: typing.Optional[VCardOrganization] = OMIT,
        categories: typing.Optional[typing.Sequence[str]] = OMIT,
        note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VcardResource:
        """
        Create a new contact card or updates an existing one with full vCard data. Contact cards can be sent [via MMS](/api-reference/messages/send-mms) as a media file.

        Parameters
        ----------
        photo : typing.Optional[str]
            Contact's photo

        id : typing.Optional[str]
            The unique identifier of the contact. This identifier is a string that always begins with the prefix `cc_`, for example: `cc_1234567890`.

        formatted_name : typing.Optional[str]
            Full display name for the vCard.

        name : typing.Optional[VCardName]
            Structured name components.

        nickname : typing.Optional[typing.Sequence[str]]
            Nicknames or aliases.

        birthday : typing.Optional[str]
            Birthday in ISO 8601 date format (YYYY-MM-DD).

        addresses : typing.Optional[typing.Sequence[VcardAddress]]
            Physical addresses.

        url : typing.Optional[str]
            Website URL.

        phones : typing.Optional[typing.Sequence[VcardPhone]]
            Phone numbers.

        emails : typing.Optional[typing.Sequence[VcardEmail]]
            Email addresses.

        timezone : typing.Optional[str]
            Timezone (e.g., "America/New_York").

        geo : typing.Optional[VCardGeo]
            Geographic coordinates.

        title : typing.Optional[str]
            Job title or position.

        role : typing.Optional[str]
            Role or function within the organization.

        organization : typing.Optional[VCardOrganization]
            Organization or company information.

        categories : typing.Optional[typing.Sequence[str]]
            Categories or tags for organizing contacts.

        note : typing.Optional[str]
            Additional notes or comments.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VcardResource
            Successfully created/updated contact card.

        Examples
        --------
        import asyncio

        from rcs import (
            AsyncPinnacle,
            VcardAddress,
            VcardEmail,
            VCardGeo,
            VCardName,
            VCardOrganization,
            VcardPhone,
        )

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.contact_card.upsert(
                id="cc_1234567890",
                formatted_name="Jane Smith",
                name=VCardName(
                    family_name="Smith",
                    given_name="Jane",
                    additional_names=["A."],
                    honorific_prefixes=["Dr."],
                    honorific_suffixes=["PhD"],
                ),
                nickname=["Janie"],
                birthday="1990-02-15",
                addresses=[
                    VcardAddress(
                        country_name="USA",
                        extended_address="Apt. 4B",
                        locality="Anytown",
                        postal_code="90210",
                        post_office_box="PO Box 123",
                        region="CA",
                        street_address="123 Main St",
                        type=["HOME", "PREF"],
                    )
                ],
                url="https://app.pinnacle.sh",
                phones=[
                    VcardPhone(
                        type=["CELL"],
                        value="+15551234567",
                    )
                ],
                emails=[
                    VcardEmail(
                        type=["INTERNET"],
                        value="jane.smith@example.com",
                    )
                ],
                timezone="America/Los_Angeles",
                geo=VCardGeo(
                    latitude=34.0522,
                    longitude=-118.2437,
                ),
                title="Engineer",
                role="Developer",
                organization=VCardOrganization(
                    name="Acme Co",
                    units=["Engineering", "R&D"],
                ),
                categories=["Friend", "Colleague"],
                note="Test contact entry",
                photo="https://fastly.picsum.photos/id/853/200/200.jpg?hmac=f4LF-tVBBnJb9PQAVEO8GCTGWgLUnxQLw44rUofE6mQ",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upsert(
            photo=photo,
            id=id,
            formatted_name=formatted_name,
            name=name,
            nickname=nickname,
            birthday=birthday,
            addresses=addresses,
            url=url,
            phones=phones,
            emails=emails,
            timezone=timezone,
            geo=geo,
            title=title,
            role=role,
            organization=organization,
            categories=categories,
            note=note,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.internal_server_error import InternalServerError
from ...errors.not_found_error import NotFoundError
from ...errors.payment_required_error import PaymentRequiredError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.campaign_submission_result import CampaignSubmissionResult
from ...types.campaign_validation_result import CampaignValidationResult
from ...types.error import Error
from ...types.extended_rcs_campaign import ExtendedRcsCampaign
from ...types.rcs_messaging_type_enum import RcsMessagingTypeEnum
from .types.rcs_agent import RcsAgent
from .types.rcs_autofill_response import RcsAutofillResponse
from .types.rcs_campaign_keywords import RcsCampaignKeywords
from .types.rcs_campaign_traffic import RcsCampaignTraffic
from .types.rcs_links import RcsLinks
from .types.rcs_use_case import RcsUseCase

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRcsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def autofill(
        self,
        *,
        additional_info: typing.Optional[str] = OMIT,
        campaign_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RcsAutofillResponse]:
        """
        Generate campaign details based off existing campaign and the brand it's connected to.

        Parameters
        ----------
        additional_info : typing.Optional[str]
            Any additional information you want to provide.

        campaign_id : typing.Optional[str]
            Unique identifier for the campaign.
            - When autofilling 10DLC campaigns, it must begin with the prefix `dlc_` (e.g., `dlc_1234567890`)
            - When autofilling Toll-Free campaigns, it must begin with the prefix `tf_` (e.g., `tf_1234567890`)
            - When autofilling RCS campaigns, it must begin with the prefix `rcs_` (e.g., `rcs_1234567890`)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RcsAutofillResponse]
            Returns autofilled RCS information.
        """
        _response = self._client_wrapper.httpx_client.request(
            "campaigns/rcs/autofill",
            method="POST",
            json={
                "additionalInfo": additional_info,
                "campaignId": campaign_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RcsAutofillResponse,
                    parse_obj_as(
                        type_=RcsAutofillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, campaign_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ExtendedRcsCampaign]:
        """
        Retrieve RCS campaign.

        Parameters
        ----------
        campaign_id : str
            Unique identifier of the RCS campaign. Must begin with the prefix `rcs_`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtendedRcsCampaign]
            Successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"campaigns/rcs/{jsonable_encoder(campaign_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtendedRcsCampaign,
                    parse_obj_as(
                        type_=ExtendedRcsCampaign,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def submit(
        self, campaign_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CampaignSubmissionResult]:
        """
        Submit your RCS campaign for approval and activation with carriers.

        Parameters
        ----------
        campaign_id : str
            Unique identifier of the RCS campaign to retrieve. Must begin with the prefix `rcs_`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CampaignSubmissionResult]
            Successfully submitted RCS campaign for approval.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"campaigns/rcs/submit/{jsonable_encoder(campaign_id)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CampaignSubmissionResult,
                    parse_obj_as(
                        type_=CampaignSubmissionResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def upsert(
        self,
        *,
        agent: typing.Optional[RcsAgent] = OMIT,
        brand: typing.Optional[str] = OMIT,
        campaign_id: typing.Optional[str] = OMIT,
        expected_agent_responses: typing.Optional[typing.Sequence[str]] = OMIT,
        links: typing.Optional[RcsLinks] = OMIT,
        use_case: typing.Optional[RcsUseCase] = OMIT,
        opt_in_terms_and_conditions: typing.Optional[str] = OMIT,
        messaging_type: typing.Optional[RcsMessagingTypeEnum] = OMIT,
        carrier_description: typing.Optional[str] = OMIT,
        keywords: typing.Optional[RcsCampaignKeywords] = OMIT,
        traffic: typing.Optional[RcsCampaignTraffic] = OMIT,
        agent_triggers: typing.Optional[str] = OMIT,
        interaction_description: typing.Optional[str] = OMIT,
        is_conversational: typing.Optional[bool] = OMIT,
        cta_language: typing.Optional[str] = OMIT,
        demo_trigger: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtendedRcsCampaign]:
        """
        Create a new RCS campaign or update an existing one.

        <Note>
        **To create a new campaign:** Omit `campaignId` — one will be generated automatically.

        **Before you start:** Create a [brand](/api-reference/brands/upsert) first — you'll need its `id` for the [`brand`](#request.body.brand) field.

        All fields are **required** unless specified otherwise, and will be validated when [submitted](/api-reference/campaigns/rcs/submit).
        </Note>

        Parameters
        ----------
        agent : typing.Optional[RcsAgent]
            Create an agent for the campaign.

        brand : typing.Optional[str]
            Unique identifier for the brand.

        campaign_id : typing.Optional[str]
            Unique identifier for the campaign.

        expected_agent_responses : typing.Optional[typing.Sequence[str]]
            List of what the agent might say to users. See the [Expected Agent Responses](/guides/campaigns/rcs-compliance#expected-agent-responses) section for requirements. <br><br>
            **Limit:** 1 to 5

        links : typing.Optional[RcsLinks]
            Legal documentation links.

        use_case : typing.Optional[RcsUseCase]
            Use case classification for the campaign.

        opt_in_terms_and_conditions : typing.Optional[str]
            Details on how opt-in is acquired. If it is done through a website or app, provide the link. See the [Opt-In Terms and Conditions](/guides/campaigns/rcs-compliance#opt-in-terms-and-conditions) section for requirements.

        messaging_type : typing.Optional[RcsMessagingTypeEnum]

        carrier_description : typing.Optional[str]
            Description of the agent's purpose, shown to carriers for approval. See the [Carrier Description](/guides/campaigns/rcs-compliance#carrier-description) section for requirements.

        keywords : typing.Optional[RcsCampaignKeywords]

        traffic : typing.Optional[RcsCampaignTraffic]

        agent_triggers : typing.Optional[str]
            Explanation of how the agent is triggered. This includes how the first message is delivered, whether messages follow a schedule or triggered by user actions, and any external triggers. See the [Agent Triggers](/guides/campaigns/rcs-compliance#agent-triggers) section for requirements.

        interaction_description : typing.Optional[str]
            Description of all agent interactions, including primary and secondary use cases. See the [Interaction Description](/guides/campaigns/rcs-compliance#interaction-description) section for requirements.

        is_conversational : typing.Optional[bool]
            Whether the agent supports conversational flows or respond to P2A messages from the users. Set to false for one-way messages from agent to user.

        cta_language : typing.Optional[str]
            Required text that appears next to the opt-in checkbox for your opt-in form. This checkbox has to be unchecked by default. See the [CTA Language](/guides/campaigns/rcs-compliance#cta-language-opt-in-disclosure) section for requirements.

        demo_trigger : typing.Optional[str]
            Instructions on how an external reviewer can trigger messages and an example flow from the agent. This is usually an inbound text message to the agent that will start a flow of messages between the agent and the user. See the [Demo Trigger](/guides/campaigns/rcs-compliance#demo-trigger) section for requirements.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtendedRcsCampaign]
            Successfully created or updated RCS campaign.
        """
        _response = self._client_wrapper.httpx_client.request(
            "campaigns/rcs",
            method="POST",
            json={
                "agent": convert_and_respect_annotation_metadata(object_=agent, annotation=RcsAgent, direction="write"),
                "brand": brand,
                "campaignId": campaign_id,
                "expectedAgentResponses": expected_agent_responses,
                "links": convert_and_respect_annotation_metadata(object_=links, annotation=RcsLinks, direction="write"),
                "useCase": convert_and_respect_annotation_metadata(
                    object_=use_case, annotation=RcsUseCase, direction="write"
                ),
                "optInTermsAndConditions": opt_in_terms_and_conditions,
                "messagingType": messaging_type,
                "carrierDescription": carrier_description,
                "keywords": convert_and_respect_annotation_metadata(
                    object_=keywords, annotation=RcsCampaignKeywords, direction="write"
                ),
                "traffic": convert_and_respect_annotation_metadata(
                    object_=traffic, annotation=RcsCampaignTraffic, direction="write"
                ),
                "agentTriggers": agent_triggers,
                "interactionDescription": interaction_description,
                "isConversational": is_conversational,
                "ctaLanguage": cta_language,
                "demoTrigger": demo_trigger,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtendedRcsCampaign,
                    parse_obj_as(
                        type_=ExtendedRcsCampaign,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def validate(
        self,
        *,
        campaign_id: str,
        additional_info: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CampaignValidationResult]:
        """
        Validate your RCS campaign configuration against carrier requirements and compliance rules.

        Parameters
        ----------
        campaign_id : str
            Unique identifier for the campaign.
            - When validating 10DLC campaigns, it must begin with the prefix `dlc_` (e.g., `dlc_1234567890`)
            - When validating toll-free campaigns, it must begin with the prefix `tf_` (e.g., `tf_1234567890`)
            - When validating RCS campaigns, it must begin with the prefix `rcs_` (e.g., `rcs_1234567890`)

        additional_info : typing.Optional[str]
            Any additional information you want to provide.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CampaignValidationResult]
            Returns validation results.
        """
        _response = self._client_wrapper.httpx_client.request(
            "campaigns/rcs/validate",
            method="POST",
            json={
                "additionalInfo": additional_info,
                "campaignId": campaign_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CampaignValidationResult,
                    parse_obj_as(
                        type_=CampaignValidationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRcsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def autofill(
        self,
        *,
        additional_info: typing.Optional[str] = OMIT,
        campaign_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RcsAutofillResponse]:
        """
        Generate campaign details based off existing campaign and the brand it's connected to.

        Parameters
        ----------
        additional_info : typing.Optional[str]
            Any additional information you want to provide.

        campaign_id : typing.Optional[str]
            Unique identifier for the campaign.
            - When autofilling 10DLC campaigns, it must begin with the prefix `dlc_` (e.g., `dlc_1234567890`)
            - When autofilling Toll-Free campaigns, it must begin with the prefix `tf_` (e.g., `tf_1234567890`)
            - When autofilling RCS campaigns, it must begin with the prefix `rcs_` (e.g., `rcs_1234567890`)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RcsAutofillResponse]
            Returns autofilled RCS information.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "campaigns/rcs/autofill",
            method="POST",
            json={
                "additionalInfo": additional_info,
                "campaignId": campaign_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RcsAutofillResponse,
                    parse_obj_as(
                        type_=RcsAutofillResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, campaign_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ExtendedRcsCampaign]:
        """
        Retrieve RCS campaign.

        Parameters
        ----------
        campaign_id : str
            Unique identifier of the RCS campaign. Must begin with the prefix `rcs_`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtendedRcsCampaign]
            Successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"campaigns/rcs/{jsonable_encoder(campaign_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtendedRcsCampaign,
                    parse_obj_as(
                        type_=ExtendedRcsCampaign,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def submit(
        self, campaign_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CampaignSubmissionResult]:
        """
        Submit your RCS campaign for approval and activation with carriers.

        Parameters
        ----------
        campaign_id : str
            Unique identifier of the RCS campaign to retrieve. Must begin with the prefix `rcs_`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CampaignSubmissionResult]
            Successfully submitted RCS campaign for approval.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"campaigns/rcs/submit/{jsonable_encoder(campaign_id)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CampaignSubmissionResult,
                    parse_obj_as(
                        type_=CampaignSubmissionResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def upsert(
        self,
        *,
        agent: typing.Optional[RcsAgent] = OMIT,
        brand: typing.Optional[str] = OMIT,
        campaign_id: typing.Optional[str] = OMIT,
        expected_agent_responses: typing.Optional[typing.Sequence[str]] = OMIT,
        links: typing.Optional[RcsLinks] = OMIT,
        use_case: typing.Optional[RcsUseCase] = OMIT,
        opt_in_terms_and_conditions: typing.Optional[str] = OMIT,
        messaging_type: typing.Optional[RcsMessagingTypeEnum] = OMIT,
        carrier_description: typing.Optional[str] = OMIT,
        keywords: typing.Optional[RcsCampaignKeywords] = OMIT,
        traffic: typing.Optional[RcsCampaignTraffic] = OMIT,
        agent_triggers: typing.Optional[str] = OMIT,
        interaction_description: typing.Optional[str] = OMIT,
        is_conversational: typing.Optional[bool] = OMIT,
        cta_language: typing.Optional[str] = OMIT,
        demo_trigger: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtendedRcsCampaign]:
        """
        Create a new RCS campaign or update an existing one.

        <Note>
        **To create a new campaign:** Omit `campaignId` — one will be generated automatically.

        **Before you start:** Create a [brand](/api-reference/brands/upsert) first — you'll need its `id` for the [`brand`](#request.body.brand) field.

        All fields are **required** unless specified otherwise, and will be validated when [submitted](/api-reference/campaigns/rcs/submit).
        </Note>

        Parameters
        ----------
        agent : typing.Optional[RcsAgent]
            Create an agent for the campaign.

        brand : typing.Optional[str]
            Unique identifier for the brand.

        campaign_id : typing.Optional[str]
            Unique identifier for the campaign.

        expected_agent_responses : typing.Optional[typing.Sequence[str]]
            List of what the agent might say to users. See the [Expected Agent Responses](/guides/campaigns/rcs-compliance#expected-agent-responses) section for requirements. <br><br>
            **Limit:** 1 to 5

        links : typing.Optional[RcsLinks]
            Legal documentation links.

        use_case : typing.Optional[RcsUseCase]
            Use case classification for the campaign.

        opt_in_terms_and_conditions : typing.Optional[str]
            Details on how opt-in is acquired. If it is done through a website or app, provide the link. See the [Opt-In Terms and Conditions](/guides/campaigns/rcs-compliance#opt-in-terms-and-conditions) section for requirements.

        messaging_type : typing.Optional[RcsMessagingTypeEnum]

        carrier_description : typing.Optional[str]
            Description of the agent's purpose, shown to carriers for approval. See the [Carrier Description](/guides/campaigns/rcs-compliance#carrier-description) section for requirements.

        keywords : typing.Optional[RcsCampaignKeywords]

        traffic : typing.Optional[RcsCampaignTraffic]

        agent_triggers : typing.Optional[str]
            Explanation of how the agent is triggered. This includes how the first message is delivered, whether messages follow a schedule or triggered by user actions, and any external triggers. See the [Agent Triggers](/guides/campaigns/rcs-compliance#agent-triggers) section for requirements.

        interaction_description : typing.Optional[str]
            Description of all agent interactions, including primary and secondary use cases. See the [Interaction Description](/guides/campaigns/rcs-compliance#interaction-description) section for requirements.

        is_conversational : typing.Optional[bool]
            Whether the agent supports conversational flows or respond to P2A messages from the users. Set to false for one-way messages from agent to user.

        cta_language : typing.Optional[str]
            Required text that appears next to the opt-in checkbox for your opt-in form. This checkbox has to be unchecked by default. See the [CTA Language](/guides/campaigns/rcs-compliance#cta-language-opt-in-disclosure) section for requirements.

        demo_trigger : typing.Optional[str]
            Instructions on how an external reviewer can trigger messages and an example flow from the agent. This is usually an inbound text message to the agent that will start a flow of messages between the agent and the user. See the [Demo Trigger](/guides/campaigns/rcs-compliance#demo-trigger) section for requirements.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtendedRcsCampaign]
            Successfully created or updated RCS campaign.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "campaigns/rcs",
            method="POST",
            json={
                "agent": convert_and_respect_annotation_metadata(object_=agent, annotation=RcsAgent, direction="write"),
                "brand": brand,
                "campaignId": campaign_id,
                "expectedAgentResponses": expected_agent_responses,
                "links": convert_and_respect_annotation_metadata(object_=links, annotation=RcsLinks, direction="write"),
                "useCase": convert_and_respect_annotation_metadata(
                    object_=use_case, annotation=RcsUseCase, direction="write"
                ),
                "optInTermsAndConditions": opt_in_terms_and_conditions,
                "messagingType": messaging_type,
                "carrierDescription": carrier_description,
                "keywords": convert_and_respect_annotation_metadata(
                    object_=keywords, annotation=RcsCampaignKeywords, direction="write"
                ),
                "traffic": convert_and_respect_annotation_metadata(
                    object_=traffic, annotation=RcsCampaignTraffic, direction="write"
                ),
                "agentTriggers": agent_triggers,
                "interactionDescription": interaction_description,
                "isConversational": is_conversational,
                "ctaLanguage": cta_language,
                "demoTrigger": demo_trigger,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtendedRcsCampaign,
                    parse_obj_as(
                        type_=ExtendedRcsCampaign,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def validate(
        self,
        *,
        campaign_id: str,
        additional_info: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CampaignValidationResult]:
        """
        Validate your RCS campaign configuration against carrier requirements and compliance rules.

        Parameters
        ----------
        campaign_id : str
            Unique identifier for the campaign.
            - When validating 10DLC campaigns, it must begin with the prefix `dlc_` (e.g., `dlc_1234567890`)
            - When validating toll-free campaigns, it must begin with the prefix `tf_` (e.g., `tf_1234567890`)
            - When validating RCS campaigns, it must begin with the prefix `rcs_` (e.g., `rcs_1234567890`)

        additional_info : typing.Optional[str]
            Any additional information you want to provide.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CampaignValidationResult]
            Returns validation results.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "campaigns/rcs/validate",
            method="POST",
            json={
                "additionalInfo": additional_info,
                "campaignId": campaign_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CampaignValidationResult,
                    parse_obj_as(
                        type_=CampaignValidationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .rich_button import RichButton


class MessageEventRcsTextContent(UniversalBaseModel):
    """
    RCS message containing longer text content with optional quick reply buttons.
    """

    type: typing.Literal["RCS_TEXT"] = pydantic.Field(default="RCS_TEXT")
    """
    Message type identifier.
    """

    id: str = pydantic.Field()
    """
    Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`. <br><br>
    To get the message details, use the [GET /messages/{id}](/api-reference/messages/get) endpoint.
    """

    quick_replies: typing_extensions.Annotated[typing.List[RichButton], FieldMetadata(alias="quickReplies")] = (
        pydantic.Field()
    )
    """
    List of interactive quick reply buttons in the message. <br><br>
    **Limit:** 10 max
    """

    text: str = pydantic.Field()
    """
    Text content of the RCS message.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

TollFreeCampaignUseCaseEnum = typing.Union[
    typing.Literal[
        "APP_NOTIFICATIONS",
        "APPOINTMENTS",
        "AUCTIONS",
        "AUTO_REPAIR_SERVICES",
        "BANK_TRANSFERS",
        "BILLING",
        "BOOKING_CONFIRMATIONS",
        "BUSINESS_UPDATES",
        "CAREER_TRAINING",
        "CHATBOT",
        "CONVERSATIONAL_ALERTS",
        "COURIER_SERVICES_DELIVERIES",
        "COVID_19_ALERTS",
        "EMERGENCY_ALERTS",
        "EVENTS_PLANNING",
        "FINANCIAL_SERVICES",
        "FRAUD_ALERTS",
        "FUNDRAISING",
        "GENERAL_MARKETING",
        "GENERAL_SCHOOL_UPDATES",
        "HEALTHCARE_ALERTS",
        "HOUSING_COMMUNITY_UPDATES",
        "HR_STAFFING",
        "INSURANCE_SERVICES",
        "JOB_DISPATCH",
        "LEGAL_SERVICES",
        "MIXED",
        "MOTIVATIONAL_REMINDERS",
        "NOTARY_NOTIFICATIONS",
        "ORDER_NOTIFICATIONS",
        "POLITICAL",
        "PUBLIC_WORKS",
        "REAL_ESTATE_SERVICES",
        "RELIGIOUS_SERVICES",
        "REPAIR_DIAGNOSTICS_ALERTS",
        "REWARDS_PROGRAM",
        "SURVEYS",
        "SYSTEM_ALERTS",
        "TWO_FA",
        "VOTING_REMINDERS",
        "WAITLIST_ALERTS",
        "WEBINAR_REMINDERS",
        "WORKSHOP_ALERTS",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .phone_number_campaign_detach_failed_item import PhoneNumberCampaignDetachFailedItem
from .phone_number_campaign_detach_phone_numbers_item import PhoneNumberCampaignDetachPhoneNumbersItem


class DetachedPhoneNumberResult(UniversalBaseModel):
    phone_numbers: typing_extensions.Annotated[
        typing.List[PhoneNumberCampaignDetachPhoneNumbersItem], FieldMetadata(alias="phoneNumbers")
    ] = pydantic.Field()
    """
    List of phone numbers.
    """

    failed: typing.List[PhoneNumberCampaignDetachFailedItem] = pydantic.Field()
    """
    List of numbers that were not failed to be detached.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ReactionResult(UniversalBaseModel):
    message_id: typing_extensions.Annotated[str, FieldMetadata(alias="messageId")] = pydantic.Field()
    """
    Unique identifier of the message. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.
    """

    reaction_message_id: typing_extensions.Annotated[str, FieldMetadata(alias="reactionMessageId")] = pydantic.Field()
    """
    Unique identifier of the message with the reaction. This identifier is a string that always begins with the prefix `msg_`, for example: `msg_1234567890`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .rcs_capability_actions import RcsCapabilityActions
from .rcs_capability_cards import RcsCapabilityCards


class RcsCapability(UniversalBaseModel):
    """
    RCS capabilities for a phone number
    """

    cards: RcsCapabilityCards
    actions: RcsCapabilityActions = pydantic.Field()
    """
    Supported button action types inside the rich cards or quick replies. Each key is a button action type that defines a specific button type in the Pinnacle API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .rcs_button_open_url_webview_mode import RcsButtonOpenUrlWebviewMode


class RcsButtonOpenUrl(UniversalBaseModel):
    """
    Button that opens a URL when tapped by the recipient.
    """

    webview_mode: typing_extensions.Annotated[
        typing.Optional[RcsButtonOpenUrlWebviewMode], FieldMetadata(alias="webviewMode")
    ] = pydantic.Field(default=None)
    """
    Controls how the URL is displayed when the button is tapped.
    
    **Default behavior:** If not specified, the URL opens in the device's default browser. If you're sending deeplinks, you should use this mode as it will open the deeplink in the native app if it is installed.
    
    **Available modes in order of size:**
    - `HALF` — Half-screen webview overlay
    - `TALL` — Tall webview overlay
    - `FULL` — Full-screen webview
    """

    payload: str = pydantic.Field()
    """
    The URL to open when the button is tapped. Must be a valid HTTP or HTTPS URL.
    """

    title: str = pydantic.Field()
    """
    Display text for the button.
    """

    metadata: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional additional data to attach to this button.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
